@@ -40,7 +40,8 @@ from barman.command_wrappers import Rsync, RsyncPgData, \
     CommandFailedException
 from barman.compression import CompressionManager, CompressionIncompatibility
 from barman.hooks import HookScriptRunner
-from barman.utils import human_readable_timedelta, mkpath, pretty_size
+from barman.utils import human_readable_timedelta, mkpath, pretty_size, \
+    fsync_dir
 from barman.config import BackupOptions
 
 
@@ -190,7 +191,12 @@ class BackupManager(object):
         previous_backup = self.get_previous_backup(backup.backup_id)
         next_backup = self.get_next_backup(backup.backup_id)
         # Remove the pgdata directory of the backup
-        self.delete_pgdata(backup)
+        try:
+            self.delete_pgdata(backup)
+        except OSError as e:
+            output.error("Failure deleting backup %s for server %s.\n%s",
+                         backup.backup_id, self.config.name, e)
+            return
         # We are deleting the first available backup
         if not previous_backup:
             # In the case of exclusive backup (default), removes any WAL
@@ -199,7 +205,7 @@ class BackupManager(object):
             # prior to the start of the backup being deleted, as they
             # might be useful to any concurrent backup started immediately
             # after.
-            remove_until = None # means to remove all WAL files
+            remove_until = None  # means to remove all WAL files
             if next_backup:
                 remove_until = next_backup
             elif BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
@@ -209,7 +215,14 @@ class BackupManager(object):
                 output.info("\t%s", name)
         # As last action, remove the basebackup directory,
         # ending the delete operation
-        self.delete_basebackup(backup)
+        try:
+            self.delete_basebackup(backup)
+        except OSError as e:
+            output.error("Failure deleting backup %s for server %s.\n%s\n"
+                         "Please manually remove the '%s' directory",
+                         backup.backup_id, self.config.name, e,
+                         backup.get_basebackup_directory())
+            return
         output.info("Done")
 
     def retry_backup_copy(self, target_function, *args, **kwargs):
@@ -397,7 +410,7 @@ class BackupManager(object):
             except FsOperationFailed:
                 output.error(
                     "Unable to connect to the target host using the command "
-                    "'%s'" % remote_command)
+                    "'%s'", remote_command)
                 output.close_and_exit()
         else:
             # if is a local recovery create a UnixLocalCommand
@@ -627,8 +640,8 @@ class BackupManager(object):
                           ':%s' % os.path.join(wal_dest, 'archive_status'))
                 except CommandFailedException:
                     output.exception(
-                        "unable to populate pg_xlog/archive_status directory",
-                        e)
+                        "unable to populate pg_xlog/archive_status"
+                        "directory: %s", e)
                     output.close_and_exit()
 
                 shutil.rmtree(status_dir)
@@ -724,9 +737,8 @@ class BackupManager(object):
                             self.config.name,
                             log=False)
             # Get the first available backup
-            first_backup_id = self.get_first_backup(BackupInfo.STATUS_ALL)
-            first_backup = self.server.get_backup(first_backup_id) \
-                    if first_backup_id else None
+            first_backup_id = self.get_first_backup(BackupInfo.STATUS_NOT_EMPTY)
+            first_backup = self.server.get_backup(first_backup_id)
             for filename in sorted(glob(
                     os.path.join(self.config.incoming_wals_directory, '*'))):
                 if not found and not verbose:
@@ -768,6 +780,7 @@ class BackupManager(object):
                 # Updates the information of the WAL archive with
                 # the latest segments
                 fxlogdb.write(wal_info.to_xlogdb_line())
+            os.fsync(fxlogdb.fileno())
         if not found and verbose:
             output.info("\tno file found", log=False)
 
@@ -805,8 +818,9 @@ class BackupManager(object):
         """
         backup_dir = backup.get_basebackup_directory()
         pg_data = os.path.join(backup_dir, 'pgdata')
-        _logger.debug("Deleting PGDATA directory: %s" % pg_data)
-        shutil.rmtree(pg_data)
+        if os.path.exists(pg_data):
+            _logger.debug("Deleting PGDATA directory: %s" % pg_data)
+            shutil.rmtree(pg_data)
 
     def delete_wal(self, wal_info):
         """
@@ -1043,15 +1057,7 @@ class BackupManager(object):
         deduplicated_size = 0
         for dirpath, _, filenames in os.walk(backup_dest):
             # execute fsync() on the containing directory
-            dir_fd = os.open(dirpath, os.O_DIRECTORY)
-            try:
-                os.fsync(dir_fd)
-            except OSError, e:
-                # On some filesystem doing a fsync on a directory
-                # raises an EINVAL error. Ignoring it is usually safe.
-                if e.errno != errno.EINVAL:
-                    raise
-            os.close(dir_fd)
+            fsync_dir(dirpath)
             # execute fsync() on all the contained files
             for filename in filenames:
                 file_path = os.path.join(dirpath, filename)
@@ -1302,15 +1308,7 @@ class BackupManager(object):
             shutil.move(srcfile, destfile)
 
         # execute fsync() on the archived WAL containing directory
-        dir_fd = os.open(destdir, os.O_DIRECTORY)
-        try:
-            os.fsync(dir_fd)
-        except OSError, e:
-            # On some filesystem doing a fsync on a directory
-            # raises an EINVAL error. Ignoring it is usually safe.
-            if e.errno != errno.EINVAL:
-                raise
-        os.close(dir_fd)
+        fsync_dir(destdir)
         # execute fsync() on the archived WAL file
         file_fd = os.open(destfile, os.O_RDONLY)
         os.fsync(file_fd)
@@ -1457,50 +1455,54 @@ class BackupManager(object):
         wal_count = label_count = history_count = 0
         # lock the xlogdb as we are about replacing it completely
         with self.server.xlogdb('w') as fxlogdb:
-            for name in sorted(os.listdir(root)):
-                # ignore the xlogdb and its lockfile
-                if name.startswith(self.server.XLOG_DB):
-                    continue
-                fullname = join(root, name)
-                if isdir(fullname):
-                    # all relevant files are in subdirectories
-                    hash_dir = fullname
-                    for wal_name in sorted(os.listdir(hash_dir)):
-                        fullname = join(hash_dir, wal_name)
-                        if isdir(fullname):
-                            _logger.warning(
-                                'unexpected directory '
-                                'rebuilding the wal database: %s',
-                                fullname)
-                        else:
-                            if xlog.is_wal_file(fullname):
-                                wal_count += 1
-                            elif xlog.is_backup_file(fullname):
-                                label_count += 1
-                            else:
+            xlogdb_new = fxlogdb.name + ".new"
+            with open(xlogdb_new, 'w') as fxlogdb_new:
+                for name in sorted(os.listdir(root)):
+                    # ignore the xlogdb and its lockfile
+                    if name.startswith(self.server.XLOG_DB):
+                        continue
+                    fullname = join(root, name)
+                    if isdir(fullname):
+                        # all relevant files are in subdirectories
+                        hash_dir = fullname
+                        for wal_name in sorted(os.listdir(hash_dir)):
+                            fullname = join(hash_dir, wal_name)
+                            if isdir(fullname):
                                 _logger.warning(
-                                    'unexpected file '
+                                    'unexpected directory '
                                     'rebuilding the wal database: %s',
                                     fullname)
-                                continue
+                            else:
+                                if xlog.is_wal_file(fullname):
+                                    wal_count += 1
+                                elif xlog.is_backup_file(fullname):
+                                    label_count += 1
+                                else:
+                                    _logger.warning(
+                                        'unexpected file '
+                                        'rebuilding the wal database: %s',
+                                        fullname)
+                                    continue
+                                wal_info = WalFileInfo.from_file(
+                                    fullname,
+                                    default_compression=default_compression)
+                                fxlogdb_new.write(wal_info.to_xlogdb_line())
+                    else:
+                        # only history files are here
+                        if xlog.is_history_file(fullname):
+                            history_count += 1
                             wal_info = WalFileInfo.from_file(
                                 fullname,
                                 default_compression=default_compression)
-                            fxlogdb.write(wal_info.to_xlogdb_line())
-                else:
-                    # only history files are here
-                    if xlog.is_history_file(fullname):
-                        history_count += 1
-                        wal_info = WalFileInfo.from_file(
-                            fullname,
-                            default_compression=default_compression)
-                        fxlogdb.write(wal_info.to_xlogdb_line())
-                    else:
-                        _logger.warning(
-                            'unexpected file '
-                            'rebuilding the wal database: %s',
-                            fullname)
-
+                            fxlogdb_new.write(wal_info.to_xlogdb_line())
+                        else:
+                            _logger.warning(
+                                'unexpected file '
+                                'rebuilding the wal database: %s',
+                                fullname)
+                os.fsync(fxlogdb_new.fileno())
+            shutil.move(xlogdb_new, fxlogdb.name)
+            fsync_dir(os.path.dirname(fxlogdb.name))
         output.info('Done rebuilding xlogdb for server %s '
                     '(history: %s, backup_labels: %s, wal_file: %s)',
                     self.config.name, history_count, label_count, wal_count)
@@ -1540,7 +1542,10 @@ class BackupManager(object):
                         # Delete the WAL segment
                         self.delete_wal(wal_info)
                         removed.append(wal_info.name)
+                fxlogdb_new.flush()
+                os.fsync(fxlogdb_new.fileno())
             shutil.move(xlogdb_new, fxlogdb.name)
+            fsync_dir(os.path.dirname(fxlogdb.name))
         return removed
 
     def validate_last_backup_maximum_age(self, last_backup_maximum_age):
