@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
 #
 # This file is part of Barman.
 #
@@ -48,12 +48,15 @@ class RetentionPolicy(object):
         self._first_backup = None
         self._first_wal = None
 
-    def report(self, context=None):
+    def report(self, source=None, context=None):
         '''Report obsolete/valid objects according to the retention policy'''
-        if context == None:
+        if context is None:
             context = self.context
+        # Overrides the list of available backups
+        if source is None:
+            source = self.server.get_available_backups(BackupInfo.STATUS_NOT_EMPTY)
         if context == 'BASE':
-            return self._backup_report()
+            return self._backup_report(source)
         elif context == 'WAL':
             return self._wal_report()
         else:
@@ -61,8 +64,9 @@ class RetentionPolicy(object):
 
     def backup_status(self, backup_id):
         '''Report the status of a backup according to the retention policy'''
+        source = self.server.get_available_backups(BackupInfo.STATUS_NOT_EMPTY)
         if self.context == 'BASE':
-            return self._backup_report()[backup_id]
+            return self._backup_report(source)[backup_id]
         else:
             return BackupInfo.NONE
 
@@ -89,7 +93,7 @@ class RetentionPolicy(object):
         pass
 
     @abstractmethod
-    def _backup_report(self):
+    def _backup_report(self, source):
         '''Report obsolete/valid backups according to the retention policy'''
         pass
 
@@ -113,7 +117,7 @@ class RetentionPolicy(object):
         """
         Output representation of the obj for JSON serialization
         """
-        return "%s %s" % (self.mode, self.value)
+        return "%s %s %s" % (self.mode, self.value, self.unit)
 
 
 class RedundancyRetentionPolicy(RetentionPolicy):
@@ -127,7 +131,7 @@ class RedundancyRetentionPolicy(RetentionPolicy):
 
     def __init__(self, context, value, server):
         super(RedundancyRetentionPolicy, self
-              ).__init__('redundancy', 'r', value, 'BASE', server)
+              ).__init__('redundancy', 'b', value, 'BASE', server)
         assert (value >= 0)
 
     def __str__(self):
@@ -136,10 +140,10 @@ class RedundancyRetentionPolicy(RetentionPolicy):
     def debug(self):
         return "Redundancy: %s (%s)" % (self.value, self.context)
 
-    def _backup_report(self):
+    def _backup_report(self, source):
         '''Report obsolete/valid backups according to the retention policy'''
         report = dict()
-        backups = self.server.get_available_backups(BackupInfo.STATUS_NOT_EMPTY)
+        backups = source
         # Normalise the redundancy value (according to minimum redundancy)
         redundancy = self.value
         if redundancy < self.server.config.minimum_redundancy:
@@ -230,10 +234,10 @@ class RecoveryWindowRetentionPolicy(RetentionPolicy):
         '''
         return datetime.now(tz.tzlocal()) - self.timedelta
 
-    def _backup_report(self):
+    def _backup_report(self, source):
         '''Report obsolete/valid backups according to the retention policy'''
         report = dict()
-        backups = self.server.get_available_backups(BackupInfo.STATUS_NOT_EMPTY)
+        backups = source
         # Map as VALID all DONE backups having end time lower than
         # the point of recoverability. The older ones
         # are classified as OBSOLETE.
@@ -319,7 +323,7 @@ class SimpleWALRetentionPolicy(RetentionPolicy):
     def debug(self):
         return "Simple WAL Retention Policy (%s)" % (self.policy)
 
-    def _backup_report(self):
+    def _backup_report(self, source):
         '''Report obsolete/valid backups according to the retention policy'''
         pass
 
