@@ -862,7 +862,7 @@ class Server(object):
             with LockFile(filename, raise_if_fail=True):
                 self.backup_manager.backup()
             # Archive incoming WALs and update WAL catalogue through cron
-            self.cron(verbose=False)
+            self.cron(verbose=False, retention_policies=False)
 
         except LockFileBusy:
             output.error("Another backup process is running")
@@ -897,9 +897,8 @@ class Server(object):
         """
         Lists all the available backups for the server
         """
-        status_filter = BackupInfo.STATUS_NOT_EMPTY
         retention_status = self.report_backups()
-        backups = self.get_available_backups(status_filter)
+        backups = self.get_available_backups(BackupInfo.STATUS_ALL)
         for key in sorted(backups.iterkeys(), reverse=True):
             backup = backups[key]
 
@@ -917,20 +916,25 @@ class Server(object):
 
     def get_backup(self, backup_id):
         """
-        Return the backup information for the given backup,
-        or None if its status is not empty
+        Return the backup information for the given backup id.
 
-        :param backup_id: the ID of the backup to return
+        If the backup_id is None or backup.info file doesn't exists,
+        it returns None.
+
+        :param str|None backup_id: the ID of the backup to return
+        :rtype: BackupInfo|None
         """
-        try:
-            backup = BackupInfo(self, backup_id=backup_id)
-            if backup.status in BackupInfo.STATUS_NOT_EMPTY:
-                return backup
+        if backup_id is None:
             return None
+        try:
+            backup_info = BackupInfo(self, backup_id=backup_id)
         except Exception, e:
             _logger.debug("Error reading backup information for %s %s: %s",
                           self.config.name, backup_id, e, exc_info=1)
             return None
+        if not os.path.exists(backup_info.filename):
+            return None
+        return backup_info
 
     def get_previous_backup(self, backup_id):
         """
@@ -1057,17 +1061,17 @@ class Server(object):
         wal_info['wals_per_second'] = 0.0
 
         for item in self.get_wal_until_next_backup(backup_info):
-                if item.name == begin:
-                    wal_info['wal_first'] = item.name
-                    wal_info['wal_first_timestamp'] = item.time
-                if item.name <= end:
-                    wal_info['wal_num'] += 1
-                    wal_info['wal_size'] += item.size
-                else:
-                    wal_info['wal_until_next_num'] += 1
-                    wal_info['wal_until_next_size'] += item.size
-                wal_info['wal_last'] = item.name
-                wal_info['wal_last_timestamp'] = item.time
+            if item.name == begin:
+                wal_info['wal_first'] = item.name
+                wal_info['wal_first_timestamp'] = item.time
+            if item.name <= end:
+                wal_info['wal_num'] += 1
+                wal_info['wal_size'] += item.size
+            else:
+                wal_info['wal_until_next_num'] += 1
+                wal_info['wal_until_next_size'] += item.size
+            wal_info['wal_last'] = item.name
+            wal_info['wal_last_timestamp'] = item.time
 
         # Calculate statistics only for complete backups
         # If the cron is not running for any reason, the required
@@ -1077,11 +1081,14 @@ class Server(object):
             # Calculate the difference between the timestamps of
             # the first WAL (begin of backup) and the last WAL
             # associated to the current backup
-            wal_info['wal_total_seconds'] = (wal_info['wal_last_timestamp'] -
-                                             wal_info['wal_first_timestamp'])
+            wal_info['wal_total_seconds'] = (
+                wal_info['wal_last_timestamp'] -
+                wal_info['wal_first_timestamp'])
             if wal_info['wal_total_seconds'] > 0:
-                wal_info['wals_per_second'] = (float(wal_info['wal_num']) /
-                                               wal_info['wal_total_seconds'])
+                wal_info['wals_per_second'] = (
+                    float(wal_info['wal_num'] +
+                          wal_info['wal_until_next_size']) /
+                    wal_info['wal_total_seconds'])
 
             # evaluation of compression ratio for basebackup WAL files
             wal_info['wal_theoretical_size'] = \
@@ -1109,19 +1116,24 @@ class Server(object):
         '''Performs a recovery of a backup'''
         return self.backup_manager.recover(backup, dest, tablespaces, target_tli, target_time, target_xid, target_name, exclusive, remote_command)
 
-    def cron(self, verbose=True):
+    def cron(self, verbose=True, wals=True, retention_policies=True):
         """
         Maintenance operations
 
         :param bool verbose: report even if no actions
+        :param bool wals: WAL archive maintenance
+        :param bool retention_policies: retention policy maintenance
         """
         filename = os.path.join(self.config.barman_home,
                                 '.%s-cron.lock' % self.config.name)
         try:
             with LockFile(filename, raise_if_fail=True, wait=True):
-                self.backup_manager.cron(verbose=verbose)
+                # Standard maintenance (WAL archive)
+                if wals:
+                    self.backup_manager.cron(verbose=verbose)
                 # Retention policy management
-                self.backup_manager.cron_retention_policy()
+                if retention_policies:
+                    self.backup_manager.cron_retention_policy()
         except LockFilePermissionDenied, e:
             output.error("Permission denied, unable to access '%s'" % e)
         except (OSError, IOError), e:
